from ustcjwxt import log, session

#### definition

gradeToScore = {
    'A+': {'interval': (95, 100), 'as-score': 98, 'as-gpa': 4.3},
    'A': {'interval': (90, 94), 'as-score': 93, 'as-gpa': 4.0},
    'A-': {'interval': (85, 89), 'as-score': 88, 'as-gpa': 3.7},
    'B+': {'interval': (82, 84), 'as-score': 84, 'as-gpa': 3.3},
    'B': {'interval': (78, 81), 'as-score': 80, 'as-gpa': 3.0},
    'B-': {'interval': (75, 77), 'as-score': 77, 'as-gpa': 2.7},
    'C+': {'interval': (72, 74), 'as-score': 74, 'as-gpa': 2.3},
    'C': {'interval': (68, 71), 'as-score': 70, 'as-gpa': 2.0},
    'C-': {'interval': (65, 67), 'as-score': 67, 'as-gpa': 1.7},
    'D+': {'interval': (64, 64), 'as-score': 65, 'as-gpa': 1.5},
    'D': {'interval': (61, 63), 'as-score': 63, 'as-gpa': 1.3},
    'D-': {'interval': (60, 60), 'as-score': 61, 'as-gpa': 1.0},
    'F': {'interval': (0, 59), 'as-score': 0, 'as-gpa': 0.0}
}

def levelToScore(level: str) -> float:
    if level in gradeToScore:
        return gradeToScore[level]['as-score']
    else:
        return float(level)

#### API for jwxt.ustc.edu.cn

def jwxtApi_getSemesters(s: session.StudentSession) -> list:
    response_semesters = s.get('https://jw.ustc.edu.cn/for-std/grade/sheet/getSemesters').json()
    return response_semesters

def jwxtApi_getGradeSheetTypes(s: session.StudentSession) -> list:
    response_gradeSheetTypes = s.get('https://jw.ustc.edu.cn/for-std/grade/sheet/getGradeSheetTypes').json()
    return response_gradeSheetTypes

def jwxtApi_getGradeList(s: session.StudentSession, trainTypeId: int = -1, semesterIds = 'default') -> list:
    if semesterIds == 'default':
        semesterIds = [semester['id'] for semester in jwxtApi_getSemesters(s)]
    if type(semesterIds) == list:
        semesterIds = ','.join([str(semesterId) for semesterId in semesterIds])
    queryParams = {
        'trainTypeId': trainTypeId if trainTypeId != -1 else None,
        'semesterIds': semesterIds
    }
    response_gradeList = s.get('https://jw.ustc.edu.cn/for-std/grade/sheet/getGradeList', params=queryParams).json()
    return response_gradeList

#### user interface

# 所有成绩
def get_gradeList_all(s: session.StudentSession, semesterIds = 'default') -> list:
    return sum([x['scores'] for x in jwxtApi_getGradeList(s, -1, semesterIds)['semesters']], [])

# 主修成绩
def get_gradeList_major(s: session.StudentSession, semesterIds = 'default') -> list:
    return sum([x['scores'] for x in jwxtApi_getGradeList(s, 1, semesterIds)['semesters']], [])

# 辅修成绩
def get_gradeList_minor(s: session.StudentSession, semesterIds = 'default') -> list:
    return sum([x['scores'] for x in jwxtApi_getGradeList(s, 2, semesterIds)['semesters']], [])

# 修读课程总数
def grade_countCourse(gradeList: list, hasGpOnly: bool = False) -> int:
    return len([grade for grade in gradeList if not hasGpOnly or grade['gp'] is not None])

# 修读课程学分总数
def grade_countCredit(gradeList: list, hasGpOnly: bool = False) -> float:
    return sum([grade['credits'] for grade in gradeList if not hasGpOnly or grade['gp'] is not None])

# 修读课程绩点总数(非均值)
def grade_countGpTotal(gradeList: list) -> float:
    return sum([grade['gp'] * grade['credits'] for grade in gradeList if grade['gp'] is not None])

# 修读课程百分制总数(非均值)
def grade_countScoreTotal(gradeList: list, weighted = True) -> float:
    if weighted:
        return sum([levelToScore(grade['score']) * grade['credits'] for grade in gradeList if grade['gp'] is not None])
    else:
        return sum([levelToScore(grade['score']) for grade in gradeList if grade['gp'] is not None])

# 修读课程平均绩点    
def grade_calcGpa(gradeList: list) -> float:
    return grade_countGpTotal(gradeList) / grade_countCredit(gradeList, True)

# 修读课程平均百分制(加权平均)
def grade_calcWeightedScore(gradeList: list) -> float:
    return grade_countScoreTotal(gradeList) / grade_countCredit(gradeList, True)

# 修读课程平均百分制(算术平均)
def grade_calcArithmeticScore(gradeList: list) -> float:
    return grade_countScoreTotal(gradeList, weighted=False) / grade_countCourse(gradeList, True)


import time
from typing import Iterable
from ustcjwxt import log
from ustcjwxt.session import StudentSession

class SingleCacheDataInfo:
    def __init__(self) -> None:
        self.reset()

    def reset(self) -> None:
        self.openTurns = None
        self.allLesson = None
        self.selectedLesson = None
        self.currentTrun = None

class CacheDataInfo:
    def __init__(self) -> None:
        self.reset()

    def reset(self) -> None:
        self.singleCache = {}
    
    def get(self, s: StudentSession) -> SingleCacheDataInfo:
        assocID = s.get_student_assocID()
        if assocID not in self.singleCache:
            self.singleCache[assocID] = SingleCacheDataInfo()
        return self.singleCache[assocID]


cacheData = CacheDataInfo()

# basic api
# 课程参数均为 courseID (int 格式), 形如 123456

def _get_openTurns(s: StudentSession, force_retrieve: bool = False) -> list:
    if not force_retrieve and cacheData.get(s).openTurns is not None:
        return cacheData.get(s).openTurns
    
    student_ID = s.get_student_ID()
    if student_ID[:2] in ['SA', 'SB']:
        log.log_info('学生类型: 研究生 (bizTypeId: 3)')
        bizTypeId = 3
    elif student_ID[:2] in ['BZ']:
        log.log_info('学生类型: 博士生 (bizTypeId: 3)')
        bizTypeId = 3
    elif student_ID[:2] in ['PB']:
        log.log_info('学生类型: 本科生 (bizTypeId: 2)')
        bizTypeId = 2
    else:
        log.log_info('学生类型: 未知 (bizTypeId: 2)')
        log.log_error('如果无法正常获取选课列表, 请联系开发者获取进一步信息')
        bizTypeId = 2
    
    url = 'https://jw.ustc.edu.cn/ws/for-std/course-select/open-turns'    
    formData = { 'bizTypeId': bizTypeId, 'studentId': s.get_student_assocID() }
    response = s.post(url, data=formData)
    cacheData.get(s).openTurns = response.json()
    return cacheData.get(s).openTurns

def _get_allLesson(s: StudentSession, force_retrieve: bool = False) -> list:
    if not force_retrieve and cacheData.get(s).allLesson is not None:
        return cacheData.get(s).allLesson
    url = 'https://jw.ustc.edu.cn/ws/for-std/course-select/addable-lessons'
    formData = { 'turnId': _get_currentTurn(s), 'studentId': s.get_student_assocID() }
    response = s.post(url, data=formData)
    cacheData.get(s).allLesson = response.json()
    for lesson in cacheData.get(s).allLesson:
        cated = []
        for teacher in lesson['teachers']:
            cated.append(teacher['nameZh'])
        lesson['teacher_cated'] = ','.join(cated)
    return cacheData.get(s).allLesson

def _get_selectedLesson(s: StudentSession, force_retrieve: bool = True) -> list:
    if not force_retrieve and cacheData.get(s).selectedLesson is not None:
        return cacheData.get(s).selectedLesson
    url = 'https://jw.ustc.edu.cn/ws/for-std/course-select/selected-lessons'
    formData = { 'turnId': _get_currentTurn(s), 'studentId': s.get_student_assocID() }
    response = s.post(url, data=formData)
    cacheData.get(s).selectedLesson = response.json()
    for lesson in cacheData.get(s).selectedLesson:
        cated = []
        for teacher in lesson['teachers']:
            cated.append(teacher['nameZh'])
        lesson['teacher_cated'] = ','.join(cated)
    return cacheData.get(s).selectedLesson

def _get_chooseCount(s: StudentSession, courseIdList) -> dict:
    isSingle = type(courseIdList) in (int, str)
    try:
        courseIdList = [courseIdList] if isSingle else list(courseIdList)
    except TypeError:
        log.log_error(f'所提供的的 courseIdList 无法转为 list')
    url_fetch = 'https://jw.ustc.edu.cn/ws/for-std/course-select/std-count'
    formData = {'lessonIds[]': courseIdList}
    response = s.post(url_fetch, data=formData).json()
    response = { int(courseId): response[courseId] for courseId in response }
    if isSingle:
        return response[int(courseIdList[0])]
    return response

def _get_lesson_byID(s: StudentSession, courseId: int) -> dict:
    allLesson = _get_allLesson(s)
    for lesson in allLesson:
        if lesson['id'] == courseId:
            return lesson
    return None

def _get_lesson_byCode(s: StudentSession, courseCode: str) -> dict:
    allLesson = _get_allLesson(s)
    for lesson in allLesson:
        if lesson['code'] == courseCode:
            return lesson
    return None

def _get_currentTurn(s: StudentSession) -> int:
    openTurns = _get_openTurns(s)
    if len(openTurns) == 0:
        log.log_error(f'当前没有开放的选课')
        return None
    if len(openTurns) == 1:
        cacheData.get(s).currentTrun = openTurns[0]['id']
    elif cacheData.get(s).currentTrun is None:
        log.log_error(f'当前开放的选课数不为 1, 请调用 ustcjwxt.course_select.set_currentTurn 以设定当前选课')
        return None
    return cacheData.get(s).currentTrun

def _send_addRequest(s: StudentSession, courseId: int) -> str:
    courseInfo = _get_lesson_byID(s, courseId)
    log.log_info(f'[+] 正在发送选课请求: {courseInfo["code"]} {courseInfo["course"]["nameZh"]} ({courseInfo["teacher_cated"]})')
    requestData = {
        'studentAssoc': s.get_student_assocID(),
        'lessonAssoc': courseId,
        'courseSelectTurnAssoc': _get_currentTurn(s),
        'scheduleGroupAssoc': '',
        'virtualCost': '0',
    }
    response_request = s.post('https://jw.ustc.edu.cn/ws/for-std/course-select/add-request', data=requestData)
    log.log_info(f'[+] 获得服务器返回信息: 【请求唯一 UUID】 {response_request.text}')
    return response_request.text

def _send_dropRequest(s: StudentSession, courseId: int) -> str:
    courseInfo = _get_lesson_byID(s, courseId)
    log.log_info(f'[+] 正在发送退课请求: {courseInfo["code"]} {courseInfo["course"]["nameZh"]} ({courseInfo["teacher_cated"]})')
    requestData = {
        'studentAssoc': s.get_student_assocID(),
        'lessonAssoc': courseId,
        'courseSelectTurnAssoc': _get_currentTurn(s),
    }
    response_request = s.post('https://jw.ustc.edu.cn/ws/for-std/course-select/drop-request', data=requestData)
    log.log_info(f'[+] 获得服务器返回信息: 【请求唯一 UUID】 {response_request.text}')
    return response_request.text

def _query_opertaionResponse(s: StudentSession, requestID: str) -> dict:
    requestData = {
        'studentId': s.get_student_assocID(),
        'requestId': requestID,
    }
    response_request = s.post('https://jw.ustc.edu.cn/ws/for-std/course-select/add-drop-response', data=requestData)
    log.log_info(f'[+] 获得服务器返回信息: {response_request.json()}')
    return response_request.json()

# user interface
# 课程参数均为 courseCode (str 格式), 形如 IS4001.01

def set_currentTurn(s: StudentSession, turnID: int) -> None:
    cacheData.get(s).currentTrun = turnID

def get_chooseCount(s: StudentSession, courseCodeList) -> dict:
    if type(courseCodeList) is str:
        return _get_chooseCount(s, get_lesson_byCode(s, courseCodeList)['id'])
    courseIdList = [get_lesson_byCode(s, courseCode)['id'] for courseCode in courseCodeList]
    return _get_chooseCount(s, courseIdList)

def check_courseAvailable(s: StudentSession, courseCode: str) -> dict:
    course = get_lesson_byCode(s, courseCode)
    return _get_chooseCount(s, course['id']) < course['limitCount']

def add_Lesson(s: StudentSession, courseCode: str, delay_ms: float = 1000) -> str:
    courseId = get_lesson_byCode(s, courseCode)['id']
    uuid = _send_addRequest(s, courseId)
    if len(uuid) == 36:
        time.sleep(delay_ms / 1000)
        for _ in log.progress_info(range(8), '正在获取选课结果'):
            ret = _query_opertaionResponse(s, uuid)
            if ret != None:
                return ret
            time.sleep(0.8)
    return { 'success': False, 'errorMessage': { 'textZh': uuid } }

def drop_Lesson(s: StudentSession, courseCode: str, delay_ms: float = 1000) -> str:
    courseId = get_lesson_byCode(s, courseCode)['id']
    uuid = _send_dropRequest(s, courseId)
    if len(uuid) == 36:
        time.sleep(delay_ms / 1000)
        for _ in log.progress_info(range(8), '正在获取选课结果'):
            ret = _query_opertaionResponse(s, uuid)
            if ret != None:
                return ret
            time.sleep(0.8)
    return { 'success': False, 'errorMessage': { 'textZh': uuid } }

def load_cache(s: StudentSession, force_retrieve: bool = True) -> None:
    _get_openTurns(s, force_retrieve)
    _get_allLesson(s, force_retrieve)
    _get_selectedLesson(s, force_retrieve)
    _get_currentTurn(s)

send_addRequest = _send_addRequest
send_dropRequest = _send_dropRequest
query_opertaionResponse = _query_opertaionResponse
get_openTurns = _get_openTurns
get_allLesson = _get_allLesson
get_selectedLesson = _get_selectedLesson
get_lesson_byID = _get_lesson_byID
get_lesson_byCode = _get_lesson_byCode
get_currentTurn = _get_currentTurn


import time
import requests
import requests.cookies
import hashlib
import base64
from Crypto.Cipher import AES
from ustcjwxt import log, request_info


class StudentSession:
    def __init__(self, username=None, password=None, session=None, smcode_hook=None):
        self.request_session = requests.Session()
        self.password_useable = False
        self.cache = dict()
        self.smcode_hook = smcode_hook
        for k, v in request_info.base_cookie.items():
            self.request_session.cookies.set(k, v)
        # login with session
        if session is not None:
            self.login_with_session(session)
        # login with username and password
        if username is not None and password is not None:
            self.login_with_password(username, password)
        elif username is not None or password is not None:
            log.log_error('username 和 password 必须同时提供')
    
    def get_cookies(self) -> requests.cookies.RequestsCookieJar:
        return self.request_session.cookies

    def get(self, url, params=None, data=None, headers=request_info.header_uaonly, **kwargs) -> requests.Response:
        response = self.request_session.get(url, headers=headers, params=params, data=data, **kwargs)
        if response.url.startswith('https://jw.ustc.edu.cn/login'):
            log.log_warning(f'get {url} redirect to {response.url}')
            if self.password_useable:
                log.log_warning('session 无效, 正在尝试重新用密码登录')
                self.clear_cache()
                if self.login_with_password(self.username, self.password):
                    response = self.request_session.get(url, headers=headers, params=params, data=data, **kwargs)
                else:
                    log.log_error('密码登录失败')
            else:
                log.log_error('session 无效')
                self.clear_cache()
        return response
    
    def post(self, url, data=None, params=None, headers=request_info.header_uaonly, content_type='', **kwargs) -> requests.Response:
        if content_type:
            headers['content-type'] = content_type + '; charset=UTF-8'
        response = self.request_session.post(url, headers=headers, params=params, data=data, **kwargs)
        if response.url.startswith('https://jw.ustc.edu.cn/login'):
            log.log_warning(f'post {url} redirect to {response.url}')
            if self.password_useable:
                log.log_warning('session 无效, 正在尝试重新用密码登录')
                self.clear_cache()
                if self.login_with_password(self.username, self.password):
                    response = self.request_session.post(url, headers=headers, params=params, data=data, **kwargs)
                else:
                    log.log_error('密码登录失败')
            else:
                log.log_error('session 无效')
                self.clear_cache()
        return response

    def clear_cache(self) -> None:
        self.request_session.cookies.clear()
        for key in request_info.base_cookie:
            self.request_session.cookies.set(key, request_info.base_cookie[key])
        self.cache = dict()

    def login_with_password(self, username: str, password: str, smcode_hook=None) -> bool:
        self.clear_cache()
        self.username = username
        self.password = password
        smcode_hook = smcode_hook or self.smcode_hook
        # 自 2024.7 之后，教务系统经历了一次彻底的更新
        # passport.ustc.edu.cn ---> id.ustc.edu.cn
        
        response = self.get('https://id.ustc.edu.cn/cas/login?service=https:%2F%2Fjw.ustc.edu.cn%2Fucas-sso%2Flogin')
        aes_key = response.text.split('<p id="login-croypto">')[1].split('</p>')[0]
        aes_key = base64.b64decode(aes_key)
        execution = response.text.split('<p id="login-page-flowkey">')[1].split('</p>')[0]
        
        log.log_debug(f'获取到一次性秘钥: aes_key = {aes_key.hex()}')
        log.log_debug(f'获取到追溯 flowkey: execution = {execution[:36+1+5]}...{execution[-8:]}')
                
        platform_info = {
            # "userAgent": request_info.header_uaonly['User-Agent'],
            # "timezone": "\"Asia/Shanghai\"",
            # "platform": "\"Win32\"",
            # "language": "\"zh-CN\"",
            # "screenResolution": "[1080,1920]"
            "cookieValue": "523fdba79644309225e1530fd7a960bc80a31cf313c7f81f9666052c25d5ecf8",
            "error_msg": "webglInfo is not defined",
            "fingerprint": "error",
        }
        

        response = self.post('https://analytics.ustc.edu.cn/fp', json=platform_info)
        # print(response.json())
        risk_value = response.json().get('responsetoken', 'error')
        log.log_debug(f'获取到 challenge token: {risk_value = } (status_code = {response.status_code})')
        
        def pad(x):
            t = 16 - len(x) % 16
            return x + bytes([t]) * t

        aes = AES.new(aes_key, mode=AES.MODE_ECB)
        encrypted_password = aes.encrypt(pad(password.encode()))
        captcha_payload = aes.encrypt(pad('{}'.encode()))
        if risk_value != 'error':
            risk_payload = aes.encrypt(pad(f'{{"token":"{risk_value}","groupId":""}}'.encode()))
        else:
            risk_payload = aes.encrypt(pad('{"error":true}'.encode()))
            

        log.log_debug(f'已生成并加密认证三元组 (password, captcha_payload, risk_payload)')

        login_form = {
            'username': username,
            'type': 'UsernamePassword',
            '_eventId': 'submit',
            'geolocation': '',
            'execution': execution,
            'captcha_code': '',
            'croypto': base64.b64encode(aes_key).decode(),
            'password': base64.b64encode(encrypted_password).decode(),
            'captcha_payload': base64.b64encode(captcha_payload).decode(),
            'risk_payload': base64.b64encode(risk_payload).decode(),
            'targetSystem': 'sso',
            'siteId': 'sourceId',
            'riskEngine': 'true'
        }
        
        response = self.post('https://id.ustc.edu.cn/cas/login', data=login_form)
        log.log_debug(f'login redirect to {response.url}')
        log.log_debug(f'login response status code {response.status_code}')
        if '<p id="current-login-type">smsLogin</p>' in response.text:
            # 检测二步验证相关设置
            log.log_info('需要二步验证')
            if smcode_hook is None:
                log.log_error('无法获取验证码, 登录失败')
                log.log_error('suggest: 请传入钩子函数 smcode_hook 以获取验证码')
                return False
            # 获取验证码
            phone_number = response.text.split('<p id="phone-number">')[1].split('</p>')[0]
            execution_2 = response.text.split('<p id="login-page-flowkey">')[1].split('</p>')[0]
            need_sms = {
                'businessNo': "0008",
                'phone': phone_number
            }
            headers = {
                'Csrf-Key': 'IZXwJDXbypKPSqpSUFKTWwAaBVBmeGYP',
                'Csrf-Value': '1c1b43a001d568b035b484d21098121c',
            }
            log.log_debug(f'获取到追溯 flowkey: execution_2 = {execution_2[:36+1+10]}...{execution_2[-7:]}')
            log.log_info(f'正在向手机 {phone_number} 发送验证码短信')
            response = self.post('https://id.ustc.edu.cn/cas/api/protected/sms/publicNoToken/sendSmsCode', json=need_sms, headers=headers)
            log.log_debug(f'二维码发送请求 | 返回: {response.text}')
            sms_value = smcode_hook()
            second_form = {
                'username': username,
                'password': sms_value,
                'type': 'smsLogin',
                '_eventId': 'submit',
                'geolocation': '',
                'execution': execution_2,
                'captcha_code': '',
                'trustDevice': 'true'
            }
            response = self.post('https://id.ustc.edu.cn/cas/login', data=second_form)
        else:
            log.log_info('不需要二步验证')
            
        if self.check_cookie_useable():
            log.log_info('登陆成功')
            self.password_useable = True
            time.sleep(1) # 应等待教务系统缓存更新
            return True
        else:
            log.log_error('username 和 password 无效, 登陆失败')
            self.password_useable = False
            self.clear_cache()
            return False

    def login_with_session(self, session: str) -> bool:
        self.clear_cache()
        self.request_session.cookies['SESSION'] = session
        if not self.check_cookie_useable():
            log.log_error('session 无效')
            self.clear_cache()
            return False
        return True
        
    def check_cookie_useable(self) -> bool:
        response = self.get('https://jw.ustc.edu.cn/my/profile')
        if response.url.startswith('https://jw.ustc.edu.cn/login'):
            return False
        return True
    
    # return binary data with jpg format(will not cache avatar)
    def get_student_avatar(self) -> bytes:
        response = self.get('https://jw.ustc.edu.cn/my/avatar')
        return response.content

    def get_student_info(self, force_retrieve = False) -> dict:
        if not force_retrieve and 'profile_info' in self.cache:
            return self.cache['profile_info']
        response = self.get('https://jw.ustc.edu.cn/my/profile')
        stuinfo_list = ['名称', '性别', '证件类型', '证件号', '生日', '政治面貌', '邮箱', '电话', '手机', '地址', '邮编', '简介']
        stuinfo = {}
        for dtype in stuinfo_list:
            specify = f'<span class="pull-left"><strong>{dtype}</strong></span>'
            p = response.text.find(specify)
            if p == -1:
                log.log_warning(f'get_student_info: {dtype} not found')
                continue
            data = response.text[p+len(specify):].split('</span>')[0].split('<span>')[-1]
            stuinfo[dtype] = data
        self.cache['profile_info'] = stuinfo
        return stuinfo
    
    def get_student_ID(self, force_retrieve = False) -> str:
        if not force_retrieve and 'student_ID' in self.cache:
            return self.cache['student_ID']
        # 从选课界面中获得学号
        response = self.get('https://jw.ustc.edu.cn/for-std/course-select')
        if response.url.startswith('https://jw.ustc.edu.cn/for-std/course-select/turns/'):
            p = response.text.find('<span class="pull-left"><strong>学号</strong></span>')
            if p != -1:
                p = response.text.find('<span>', p + 50) + 6
                q = response.text.find('</span>', p)
                student_ID = response.text[p:q]
                self.cache['student_ID'] = student_ID
                return student_ID
        log.log_warning('未能在选课界面中找到学号')
        # 从课表页面获取学号
        courseTablePage = self.get(f'https://jw.ustc.edu.cn/for-std/course-table/info/{self.get_student_assocID()}').text
        p = courseTablePage.find('<h2 class="info-title">')
        if p != -1:
            log.log_debug('从课表页面成功获取学号')
            student_ID = courseTablePage[p:].split('(')[1].split(')')[0]
            self.cache['student_ID'] = student_ID
            return student_ID
        log.log_warning('未能在课表页面找到学号')
        # 从 cookie 中获得学号
        if 'uc' in self.request_session.cookies:
            log.log_debug('从 cookies 中成功获取学号')
            student_ID = self.request_session.cookies['uc']
            self.cache['student_ID'] = student_ID
            return student_ID
        log.log_warning('未能在 cookies 中找到学号')
        # 均失败
        log.log_warning('未能获得学号')
        return ''
    
    def get_student_assocID(self, force_retrieve = False) -> int:
        if not force_retrieve and 'assocID' in self.cache:
            return self.cache['assocID']
        response = self.get('https://jw.ustc.edu.cn/for-std/course-select')
        if response.url.startswith('https://jw.ustc.edu.cn/for-std/course-select/turns/'):
            assocID = int(response.url.split('/')[-1])
            log.log_info(f'学生唯一 assocID 获取成功: {assocID}')
            self.cache['assocID'] = assocID
            return assocID
        log.log_error(f'学生唯一 assocID 获取失败')
        return -1


base_cookie = {
    'user_locale': 'zh',
    'fine_remember_login': '-1',
    'SVRNAME': 'teacher1',
}

header_urlencoded = {
    'origin': 'https://jw.ustc.edu.cn',
    'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
    'x-requested-with': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}

header_json = {
    'origin': 'https://jw.ustc.edu.cn',
    'content-type': 'application/json; charset=UTF-8',
    'x-requested-with': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}

header_uaonly = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.114514.2025 Safari/537.36'
}


import logging, sys, datetime


logger = logging.Logger('ustcjwxt', level="INFO")

def set_logger_level(level):
    logger.setLevel(level)

def set_logger_file(filename, **kwargs):
    handler = logging.FileHandler(filename, **kwargs)
    logger.addHandler(handler)

def set_logger_format(fmt = "%(asctime)s [%(levelname)s] %(message)s", datefmt = "%Y-%m-%d %H:%M:%S", **kwargs):
    formatter = logging.Formatter(fmt, datefmt, **kwargs)
    for handler in logger.handlers:
        handler.setFormatter(formatter)

def log_debug(msg):
    logger.debug(msg)

def log_info(msg):
    logger.info(msg)

def log_warning(msg):
    logger.warning(msg)

def log_error(msg):
    logger.error(msg)


set_logger_file('/dev/stderr' if sys.platform == 'linux' else 'CON', mode='w')
set_logger_format()



strf_time = lambda x: x.strftime("%Y-%m-%d %H:%M:%S")
strf_current_time = lambda: strf_time(datetime.datetime.now())
class progress_info:
    def __init__(self, rg, info):
        self.start_iter = False
        self.it = iter(rg)
        self.cnt = 0
        self.info = info
    
    def __iter__(self):
        self.start_iter = True
        return self
    
    def __next__(self):
        try:
            val = next(self.it)
            self.cnt += 1
            print(f'{strf_current_time()} [INFO] {self.info}{self.cnt * "."}', file=sys.stderr, end='\r')
            return val
        except StopIteration:
            raise StopIteration
        
    def __del__(self):
        print()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ustcjwxt",
    version="v0.2.1",
    author="brealid",
    author_email="brealid@mail.ustc.edu.cn",
    description="ustc jwxt api, for study only",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/brealid/ustcjwxt-python-lib",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=[
        "requests",
        "pycryptodome"
    ]
)